@@ -40,6 +40,8 @@
 #' @template seealso-generate_quantity
 #' @seealso \code{\link{shinystan-metadata}} to view or change metadata
 #'   associated with a \code{shinystan} object.
+#'   
+#' @template reference-muth
 #' 
 shinystan <- setClass(
   Class = "shinystan",
@@ -570,7 +572,7 @@ setMethod(
       model_code = rstan::get_stancode(X),
       misc = list(
         max_td = .rstan_max_treedepth(X),
-        stan_method = .stan_args(X, "method"),
+        stan_method = .stan_method(X),
         stan_algorithm = .stan_algorithm(X),
         sso_version = utils::packageVersion("shinystan")
       )
@@ -615,6 +617,22 @@ setMethod(
   isTRUE("engine" %in% names(.stan_args(x)))
 }
 
+# Check if model was restored from a CSV file generated by rstan
+# @param x stanfit object
+.from_rstan_csv <- function(x) {
+  isTRUE("sampler_t" %in% names(.stan_args(x)))
+}
+
+#Get the stan method (variatinal, sampling,optimization)
+.stan_method <- function(x) {
+  if(.from_rstan_csv(x)) {
+    "sampling" #I assume there is no way to generate sample file withou sampling
+  } else {
+    .stan_args(x, "method")
+  }
+}
+
+
 # Check if model fit using variational algorithm
 # @param x stanfit object
 .used_vb <- function(x) {
@@ -624,8 +642,14 @@ setMethod(
 # Check which algorithm was used to fit model
 # @param x stanfit object
 .stan_algorithm <- function(x) {
-  algo <- if (.from_cmdstan(x))
-    toupper(.stan_args(x, "engine")) else .stan_args(x, "algorithm")
+  algo <- if (.from_cmdstan(x)) {
+    toupper(.stan_args(x, "engine"))
+  } else if (.from_rstan_csv(x)) {
+    #When reading the CSV, the engine is of the form NUTS(diag_e) - getting rid of the brackets here
+    gsub("\\([^)]*\\)","",.stan_args(x, "sampler_t"))
+  } else {
+    .stan_args(x, "algorithm")
+  }
   
   if (.used_vb(x) || !(algo %in% c("NUTS", "HMC")))
     warning("Many features are only available for models fit using
@@ -689,11 +713,11 @@ setMethod(
 .rstan_max_treedepth <- function(x) {
   cntrl <- .stan_args(x, "control")
   if (is.null(cntrl)) {
-    max_td <- 11
+    max_td <- 10
   } else {
     max_td <- cntrl$max_treedepth
     if (is.null(max_td))
-      max_td <- 11
+      max_td <- 10
   }
   max_td
 }
@@ -843,3 +867,67 @@ setMethod(
   
   pp_check_plots
 }
+
+
+# as.shinystan (CmdStanMCMC) -----------------------------------------------
+as.shinystan_cmdstanr <- function(X,
+                                  pars = NULL,
+                                  model_name = NULL,
+                                  note = NULL,
+                                  ...) {
+  check_suggests("cmdstanr")
+  check_suggests("posterior")
+  if (is.null(model_name) && !is.null(X$runset)) {
+    model_name <- X$runset$model_name()
+  } else {
+    model_name <- "unnamed model"
+  }
+  
+  if (X$metadata()$save_warmup == 0) {
+    draws <- unclass(X$draws(pars))
+    sampler_diagnostics <- X$sampler_diagnostics()
+    n_warmup <- 0
+  } else {
+    draws <- unclass(X$draws(pars, inc_warmup = TRUE))
+    sampler_diagnostics <- X$sampler_diagnostics(inc_warmup = TRUE)
+    n_warmup <- X$metadata()$iter_warmup
+  }
+  
+  sampler_params <- list()
+  for (j in seq_len(dim(sampler_diagnostics)[2])) {
+    sampler_params[[j]] <- posterior::as_draws_matrix(sampler_diagnostics[, j ,])
+  }
+  
+  as.shinystan(
+    draws,
+    model_name = model_name,
+    warmup = n_warmup,
+    param_dims = X$metadata()$stan_variable_dims,
+    model_code = NULL,
+    note = note,
+    sampler_params = sampler_params, 
+    algorithm = "NUTS",
+    max_treedepth = X$metadata()$max_treedepth
+  )
+}
+
+setOldClass("CmdStanMCMC")
+#' @describeIn as.shinystan Create a \code{shinystan} object from a 
+#'   \code{CmdStanMCMC} object (\pkg{cmdstanr}).
+#' 
+setMethod(
+  "as.shinystan",
+  signature = "CmdStanMCMC",
+  definition = as.shinystan_cmdstanr
+)
+
+setOldClass("CmdStanMCMC_CSV")
+#' @describeIn as.shinystan Create a \code{shinystan} object from a
+#'   \code{CmdStanMCMC_CSV} object created using
+#'   \code{cmdstanr::as_cmdstan_fit()} (\pkg{cmdstanr}).
+#' 
+setMethod(
+  "as.shinystan",
+  signature = "CmdStanMCMC_CSV",
+  definition = as.shinystan_cmdstanr
+)
