@@ -16,6 +16,13 @@ navlistPanel(
     hr(),
     stan_manual()
   ),
+  tabPanel(
+    "energy",
+    withMathJax(),
+    includeHTML("html/energy.html"),
+    hr(),
+    stan_manual()
+  ),
   tabPanel(
     "stepsize",
     withMathJax(),
